@@ -218,31 +218,44 @@ class PandasSpecialEngine:
             raise HasNoSchemaSystemException('Cannot create schemas for given SQL flavor '
                                              '(AFAIK only PostgreSQL has this feature)')
 
-    def schema_exists(self) -> bool:
+    # DDL related operations
+    # in these methods we will add a `connection` parameter for passing a different type
+    # of connection. This will be very useful for asynchronous connections.
+    # AFAIK DDL operations do not support asynchronous execution. In the sqlalchemy
+    # guide as well as here we we'll use sync methods with the async connections
+    # using `connection.run_sync`
+    def schema_exists(self, connection=None) -> bool:
+        """
+        Returns True if the PostgreSQL defined in given instance
+        of PandasSpecialEngine exists else returns False.
+        """
         self._raise_no_schema_feature()
+        con = self.connection if connection is None else connection
         if _sqla_gt14():
-            insp = sa.inspect(self.connection)
+            insp = sa.inspect(con)
             return self.schema in insp.get_schema_names()
         else:
-            return self.connection.dialect.has_schema(self.connection, self.schema)
+            return con.dialect.has_schema(con, self.schema)
 
-    def table_exists(self) -> bool:
+    def table_exists(self, connection=None) -> bool:
         """
         Returns True if the table defined in given instance
         of PandasSpecialEngine exists else returns False.
         """
-        insp = sa.inspect(self.connection)
+        con = self.connection if connection is None else connection
+        insp = sa.inspect(con)
         if _sqla_gt14():
             return insp.has_table(schema=self.schema, table_name=self.table.name)
         else:
             # this is not particularly efficient but AFAIK it's the best we can do at connection level
             return self.table.name in insp.get_table_names(schema=self.schema)
 
-    def create_schema_if_not_exists(self):
+    def create_schema_if_not_exists(self, connection=None):
         """
         Creates the schema defined in given instance of
         PandasSpecialEngine if it does not exist.
         """
+        # verify schema can be created
         self._raise_no_schema_feature()
         if self.schema is None:
             raise AssertionError('Cannot create schema because it is None. '
@@ -251,35 +264,38 @@ class PandasSpecialEngine:
                                  '(AFAIK only PostgreSQL has this feature) and retry your operation '
                                  'after setting the default schema `public` yourself '
                                  '(if that is the schema you wish to use).')  # pragma: no cover
-        if not self.schema_exists():
-            self.connection.execute(CreateSchema(self.schema))
+        # acquire connection and create schema
+        con = self.connection if connection is None else connection
+        if not self.schema_exists(connection=con):
+            con.execute(CreateSchema(self.schema))
 
-    def create_table_if_not_exists(self):
+    def create_table_if_not_exists(self, connection=None):
         """
         Creates the table generated in given instance of
         PandasSpecialEngine if it does not exist.
         """
-        self.table.create(checkfirst=True)
+        con = self.connection if connection is None else connection
+        self.table.create(bind=con, checkfirst=True)
 
-    def get_db_columns_names(self) -> List[str]:
+    def get_db_columns_names(self, connection=None) -> List[str]:
         """
         Gets the column names of the SQL table defined
         in given instance of PandasSpecialEngine.
         """
+        con = self.connection if connection is None else connection
         if _sqla_gt14():
-            insp = sa.inspect(self.connection)
+            insp = sa.inspect(con)
             columns_info = insp.get_columns(schema=self.schema, table_name=self.table.name)
         else:
-            columns_info = self.connection.dialect.get_columns(connection=self.connection,
-                                                               schema=self.schema,
-                                                               table_name=self.table.name)
+            columns_info = con.dialect.get_columns(connection=con, schema=self.schema,
+                                                   table_name=self.table.name)
         db_columns_names = [col_info["name"] for col_info in columns_info]
         # handle case of SQlite where no errors are raised in case of a missing table
         # but instead 0 columns are returned by sqlalchemy
         assert len(db_columns_names) > 0
         return db_columns_names
 
-    def add_new_columns(self):
+    def add_new_columns(self, connection=None):
         """
         Adds columns present in df but not in the SQL table
         for given instance of PandasSpecialEngine.
@@ -288,8 +304,9 @@ class PandasSpecialEngine:
         -----
         Sadly, it seems that we cannot create JSON columns.
         """
+        con = self.connection if connection is None else connection
         # get column names in db
-        db_columns = self.get_db_columns_names()
+        db_columns = self.get_db_columns_names(connection=con)
         # create deepcopies of the column because we are going to unbound
         # them for the table model (otherwise alembic would think we add
         # a column that already exists in the database)
@@ -300,7 +317,7 @@ class PandasSpecialEngine:
                                                   "You'll have to update your table primary key or change your "
                                                   "df index")
 
-        ctx = MigrationContext.configure(self.connection)
+        ctx = MigrationContext.configure(con)
         op = Operations(ctx)
         for col in cols_to_add:
             col.table = None  # Important! unbound column from table
@@ -308,7 +325,7 @@ class PandasSpecialEngine:
             log(f"Added column {col} (type: {col.type}) in table {self.table.name} "
                 f'(schema="{self.schema}")')
 
-    def get_db_table_schema(self) -> Table:
+    def get_db_table_schema(self, connection=None) -> Table:
         """
         Gets the sqlalchemy table model for the SQL table
         defined in given PandasSpecialEngine (using schema and
@@ -316,10 +333,10 @@ class PandasSpecialEngine:
         """
         table_name = self.table.name
         schema = self.schema
-        connection = self.connection
+        con = self.connection if connection is None else connection
 
-        metadata = MetaData(bind=connection, schema=schema)
-        metadata.reflect(bind=connection, schema=schema, only=[table_name])
+        metadata = MetaData(bind=con, schema=schema)
+        metadata.reflect(bind=con, schema=schema, only=[table_name])
         namespace = table_name if schema is None else f'{schema}.{table_name}'
         db_table = metadata.tables[namespace]
         return db_table
@@ -347,7 +364,7 @@ class PandasSpecialEngine:
                 empty_columns.append(col)
         return empty_columns
 
-    def adapt_dtype_of_empty_db_columns(self):
+    def adapt_dtype_of_empty_db_columns(self, empty_db_columns=None, connection=None, db_table=None):
         """
         Changes the data types of empty columns in the SQL table defined
         in given instance of a PandasSpecialEngine.
@@ -355,9 +372,14 @@ class PandasSpecialEngine:
         This should only happen in case of data type mismatches.
         This means with columns for which the sqlalchemy table
         model for df and the model for the SQL table have different data types.
+
+        The parameters `empty_db_columns` and `db_table` allow us to avoid synchronous operations
+        when passing an asynchronous connection via the `connection` parameter.
+        See async variant of this method: `aadapt_dtype_of_empty_db_columns`
         """
-        empty_db_columns = self.get_empty_columns()
-        db_table = self.get_db_table_schema()
+        con = self.connection if connection is None else connection
+        empty_db_columns = self.get_empty_columns() if empty_db_columns is None else empty_db_columns
+        db_table = self.get_db_table_schema() if db_table is None else db_table
         # if column does not have value in db and there are values
         # in the frame then change the column type if needed
         for col in empty_db_columns:
@@ -365,8 +387,8 @@ class PandasSpecialEngine:
             if col.name not in self.df.columns:
                 continue
             # check same type
-            orig_type = db_table.columns[col.name].type.compile(self.connection.dialect)
-            dest_type = self.table.columns[col.name].type.compile(self.connection.dialect)
+            orig_type = db_table.columns[col.name].type.compile(con.dialect)
+            dest_type = self.table.columns[col.name].type.compile(con.dialect)
             # remove character count e.g. "VARCHAR(50)" -> "VARCHAR"
             orig_type = RE_CHARCOUNT_COL_TYPE.sub('', orig_type)
             dest_type = RE_CHARCOUNT_COL_TYPE.sub('', dest_type)
@@ -386,14 +408,14 @@ class PandasSpecialEngine:
                 # (SQLite does not support data type alteration)
                 if self._db_type == 'sqlite':
                     raise ValueError('SQlite does not support column data type alteration!')
-                ctx = MigrationContext.configure(self.connection)
+                ctx = MigrationContext.configure(con)
                 op = Operations(ctx)
                 new_col = self.table.columns[col.name]
                 # check if postgres (in which case we have to use "using" syntax
                 # to alter columns data types)
                 if self._db_type == 'postgres':
-                    escaped_col = str(new_col.compile(dialect=self.connection.dialect))
-                    compiled_type = new_col.type.compile(dialect=self.connection.dialect)
+                    escaped_col = str(new_col.compile(dialect=con.dialect))
+                    compiled_type = new_col.type.compile(dialect=con.dialect)
                     alter_kwargs = {'postgresql_using':f'{escaped_col}::{compiled_type}'}
                 else:
                     alter_kwargs = {}
@@ -515,6 +537,62 @@ class PandasSpecialEngine:
         for chunk in chunks:
             yield upq.execute(db_type=self._db_type, values=chunk, if_row_exists=if_row_exists)
 
+    # ASYNC VARIANTS of methods above that we will prefix with "a"
+    # There is going to be some unfortunate repetition but I don't see a better way yet
+    # Note: I know I don't need to use lambdas here but that is a precaution to make sure the `connection` kwarg
+    # is here
+    async def atable_exists(self):
+        return await self.connection.run_sync(lambda connection: self.table_exists(connection=connection))
+
+    async def aschema_exists(self):
+        return await self.connection.run_sync(lambda connection: self.schema_exists(connection=connection))
+
+    async def acreate_table_if_not_exists(self):
+        await self.connection.run_sync(lambda connection: self.create_table_if_not_exists(connection=connection))
+
+    async def acreate_schema_if_not_exists(self):
+        await self.connection.run_sync(lambda connection: self.create_schema_if_not_exists(connection=connection))
+
+    async def aadd_new_columns(self):
+        await self.connection.run_sync(lambda connection: self.add_new_columns(connection=connection))
+
+    async def aget_empty_columns(self) -> list:
+        db_table = await self.connection.run_sync(lambda connection: self.get_db_table_schema(connection=connection))
+        empty_columns = []
+        for col in db_table.columns:
+            stmt = select(from_obj=db_table,
+                          columns=[col],
+                          whereclause=col.isnot(None)).limit(1)
+            proxy = await self.connection.execute(stmt)
+            results = proxy.fetchall()
+            if results == []:
+                empty_columns.append(col)
+        return empty_columns
+
+    async def aadapt_dtype_of_empty_db_columns(self):
+        empty_db_columns = await self.aget_empty_columns()
+        db_table = await self.connection.run_sync(lambda connection: self.get_db_table_schema(connection=connection))
+        ddl_func = lambda connection: self.adapt_dtype_of_empty_db_columns(connection=connection,
+                                                                           empty_db_columns=empty_db_columns,
+                                                                           db_table=db_table)
+        await self.connection.run_sync(ddl_func)
+
+    async def aupsert(self, if_row_exists:str, chunksize:int=10000):
+        assert if_row_exists in ('ignore', 'update')
+        values = self._get_values_to_insert()
+        chunks = self._create_chunks(values=values, chunksize=chunksize)
+        upq = UpsertQuery(connection=self.connection, table=self.table)
+        for chunk in chunks:
+            await upq.execute(db_type=self._db_type, values=chunk, if_row_exists=if_row_exists)
+
+    async def aupsert_yield(self, if_row_exists:str, chunksize:int=10000):
+        assert if_row_exists in ('ignore', 'update')
+        values = self._get_values_to_insert()
+        chunks = self._create_chunks(values=values, chunksize=chunksize)
+        upq = UpsertQuery(connection=self.connection, table=self.table)
+        for chunk in chunks:
+            yield await upq.execute(db_type=self._db_type, values=chunk, if_row_exists=if_row_exists)
+
     def __repr__(self):
         text = f"""PandasSpecialEngine (id {id(self)}, hexid {hex(id(self))})
                    * connection: {self.connection}
