@@ -5,27 +5,25 @@ import pytest
 from math import floor
 from sqlalchemy import VARCHAR
 
-from pangres import upsert
+# local imports
 from pangres.helpers import _sqlite_gt3_32_0
 from pangres.examples import _TestsExampleTable
-from pangres.tests.conftest import drop_table_for_test, drop_table
+from pangres.tests.conftest import (drop_table_for_test, drop_table,
+                                    is_async_sqla_obj, TableNames, upsert_or_aupsert)
+
+
 # -
 
 
-# # Config
-
-table_name = 'test_speed'
-
-
 # # Helpers
 
 # +
 def create_or_upsert_with_pangres(engine, schema, if_row_exists, df, chunksize, **kwargs):
     # MySQL does not want flexible text length in indices/PK
     dtype={'profileid':VARCHAR(10)} if 'mysql' in engine.dialect.dialect_description else None
-    upsert(con=engine, df=df, schema=schema, chunksize=chunksize,
-           table_name=table_name, if_row_exists=if_row_exists,
-           dtype=dtype, **kwargs)
+    upsert_or_aupsert(con=engine, df=df, schema=schema, chunksize=chunksize,
+                      table_name=TableNames.BENCHMARK, if_row_exists=if_row_exists,
+                      dtype=dtype, **kwargs)
 
 
 def create_with_pandas(engine, schema, df):
@@ -41,7 +39,7 @@ def create_with_pandas(engine, schema, df):
         chunksize = None
 
     # create table
-    df.to_sql(con=engine, schema=schema, name=table_name, method='multi',
+    df.to_sql(con=engine, schema=schema, name=TableNames.BENCHMARK, method='multi',
               chunksize=chunksize, dtype=dtype)
 
 
@@ -59,8 +57,12 @@ pytest_params = dict(argnames='nb_rows, rounds, iterations', argvalues=[[10, 5,
 
 @pytest.mark.parametrize('library', ['pandas', 'pangres'])
 @pytest.mark.parametrize(**pytest_params)
-@drop_table_for_test(table_name=table_name)
+@drop_table_for_test(table_name=TableNames.BENCHMARK)
 def test_create_and_insert_speed(engine, schema, benchmark, library, nb_rows, rounds, iterations):
+    # skip async engines with pandas
+    if is_async_sqla_obj(engine) and library == 'pandas':
+        pytest.skip('async engines will not work with pandas')
+
     # get a df
     # we don't test JSON as this is problematic with pandas
     df = _TestsExampleTable.create_example_df(nb_rows=nb_rows).drop(columns=['favorite_colors'])
@@ -70,8 +72,14 @@ def test_create_and_insert_speed(engine, schema, benchmark, library, nb_rows, ro
                                                               df=df, chunksize=nb_rows),
               'pandas':lambda: create_with_pandas(engine=engine, schema=schema, df=df)}
 
-    benchmark.pedantic(switch[library], setup=lambda: drop_table(engine=engine, schema=schema, table_name=table_name),
-                       rounds=rounds, iterations=iterations)
+    try:
+        benchmark.pedantic(switch[library], setup=lambda: drop_table(engine=engine, schema=schema,
+                                                                     table_name=TableNames.BENCHMARK),
+                           rounds=rounds, iterations=iterations)
+    except NotImplementedError as e:
+        if 'not implemented for SQLAlchemy 2' in str(e):
+            pytest.skip('in Python 3.6 there is some kind of problem with engines created with '
+                        '`future=True` flag and pandas')
 
 
 # ## Upsert overwrite speed
@@ -80,10 +88,14 @@ def test_create_and_insert_speed(engine, schema, benchmark, library, nb_rows, ro
 @pytest.mark.parametrize('library', ['pangres'])
 @pytest.mark.parametrize('if_row_exists', ['update', 'ignore'])
 @pytest.mark.parametrize(**pytest_params)
-@drop_table_for_test(table_name=table_name)
+@drop_table_for_test(table_name=TableNames.BENCHMARK)
 def test_upsert_speed(engine, schema, benchmark, library, nb_rows, rounds, iterations, if_row_exists):
     assert library == 'pangres'  # in case pandas changes and we forget to update the tests
 
+    # skip async engines with pandas
+    if is_async_sqla_obj(engine) and library == 'pandas':
+        pytest.skip('async engines will not work with pandas')
+
     # get a df
     df = _TestsExampleTable.create_example_df(nb_rows=nb_rows).drop(columns=['favorite_colors'])
 
